% Simulation for the joint Lucy-Richardson reconstruction of the patterned
% illumination microscopy data
%
% Authors: Nadya Chakrova, Bernd Rieger, Sjoerd Stallinga
% 2015
%
% (C) Copyright 2015               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands

clear all;
close all;

% switch cases between random ('PRP'), multispot ('MSP') and line ('SIM') illumination patterns
illumination_pattern = 'MSP';

% add noise - YES/NO
Poisson_noise_on = 1;
Read_out_noise_on = 1;

% input parameters:
K = 144; % number of patterns in a multi-spot scan
S = 12; % pitch of the square pattern
NA = 1.4; % NA of the objective
lambda_ex = 488; % excitation wavelength in nm
lambda_em = 520; % emission wavelength in nm
N_iter = 200; % number of jRL iterations
readout_noise_std = 1.9; % sCMOS noise rms = 1.9e
N_photons = 25000; % max number of accumulated photons per pixel

%-----------------------------------
% Generate the Sample
%-----------------------------------
Sample = double(imread('resolutionchart512.tif', 'tif'));
Sample = N_photons.*Sample./max(Sample(:));
N = size(Sample,1); % number of pixels
deltax = 1/8.*lambda_em/2/NA; % pixel size in nm
L = N*deltax; % image size in nm

%--------------------------------------------
% Generate illumination patterns I at the DMD 
%--------------------------------------------
I = generate_illum_patterns(N, S, K, illumination_pattern);

%-----------------------------------
% Generate the PSF (vectorial)
%-----------------------------------
% PSF parameters: NA, refractive indices of medium, cover slip, immersion fluid,
% wavelength (in nm), nominal emitter position (in nm) with z-position from
% cover slip-medium interface, spot footprint (in nm), axial range (in nm),
% sampling in pupil with (even), sampling in image plane (odd), sampling in
% axial direction, emission (1) or excitation (0) light path.
disp('Generating vectorial PSF...');
refmed = 1.5;
refcov = 1.5;
refimm = 1.5;
xemit = 0;
yemit = 0;
zemit = 0;
xyrange = L/2;
zrange = 0;
Npupil = N;
Mimage = N;
Maxial = 1;
lightpath_ex = 'excitation';
lightpath_em = 'emission';

% aberration parameters
defocus = 0; horverast = 0; diagast = 0; xcoma = 0; ycoma = 0; sphab = 0;

parameters_ex = struct('NA', NA, 'refmed', refmed, 'refcov', refcov, 'refimm', refimm, 'lambda', lambda_ex, 'xemit', xemit, 'yemit', yemit,...
    'zemit', zemit, 'xyrange', xyrange, 'zrange', zrange, 'Npupil', Npupil, 'Mimage', Mimage, 'Maxial', Maxial, 'lightpath', lightpath_ex);

parameters_em = struct('NA', NA, 'refmed', refmed, 'refcov', refcov, 'refimm', refimm, 'lambda', lambda_em, 'xemit', xemit, 'yemit', yemit,...
    'zemit', zemit, 'xyrange', xyrange, 'zrange', zrange, 'Npupil', Npupil, 'Mimage', Mimage, 'Maxial', Maxial, 'lightpath', lightpath_em);

aberrations = struct('defocus', defocus, 'horverast', horverast, 'diagast', diagast, 'xcoma', xcoma, 'ycoma', ycoma, 'sphab', sphab);

FieldMatrix_ex = get_field_matrix(parameters_ex,aberrations);
FieldMatrix_em = get_field_matrix(parameters_em,aberrations);

% polarization parameters: emitter/absorber dipole orientation (characterized by angles
% pola and azim), detection/illumination polarization in objective lens
% back aperture (characterized by angles alpha and beta).

pola = 0*pi/180;
azim = 0*pi/180;
polarizationpupil = 0;
alpha = 0;
beta = 0; 

Pol_parameters = struct('pola', pola, 'azim', azim, 'polarizationpupil', polarizationpupil, 'alpha', alpha, 'beta', beta, 'lightpath', lightpath_ex);

[FreePSF_ex,FixedPSF] = get_psfs(FieldMatrix_ex, Pol_parameters);
[FreePSF_em,FixedPSF] = get_psfs(FieldMatrix_em, Pol_parameters);

FreePSF_ex = FreePSF_ex./max(FreePSF_ex(:));
FreePSF_em = FreePSF_em./max(FreePSF_em(:));

%-----------------------------------
% Generate the OTFs
%-----------------------------------
OTF_ex = abs(ft2(FreePSF_ex));
OTF_em = abs(ft2(FreePSF_em));

OTF_ex = OTF_ex./max(OTF_ex(:));
OTF_em = OTF_em./max(OTF_em(:));

%-----------------------------------------------------
% Generate illumination patterns P at the sample plane 
%-----------------------------------------------------
disp('Generating illumination patterns...');
P = zeros(N,N,K);
for i = 1:K
    P(:,:,i) = ift2(ft2(I(:,:,i)).*OTF_ex);
end
P = real(P);

%------------------------------------------
% Generate data, add poisson/gaussian noise
%------------------------------------------
disp('Generating raw data...');
D = zeros(N,N,K);
for i = 1:K
    D(:,:,i) = ift2(ft2(P(:,:,i).*Sample).*OTF_em);
end
D = real(D);

wf = ift2(ft2(Sample).*OTF_em);
wf = real(wf);

if Poisson_noise_on
    D = 1e12*imnoise(1e-12*D,'poisson');
    wf = 1e12*imnoise(1e-12*wf,'poisson');
end

if Read_out_noise_on
    wf = wf + randn(size(wf)).*readout_noise_std;
    for i = 1:K
        D(:,:,i) = D(:,:,i) + randn(N,N).*readout_noise_std;
    end
end


%---------------------------------------------------------------------
%------------------------Reconstruction by----------------------------
%---------------joint Lucy-Richardson deconvolution-------------------
%---------------------------------------------------------------------
disp('Reconstruction using joint Richardson-Lucy...');
M = zeros(size(D));
r = zeros(size(D));
abs_error = zeros(1,N_iter);

% initialize the object
Ig = wf./max(wf(:));

    for n=1:N_iter 
%         disp(['joint RL iteration ' num2str(n)]); 
        S = zeros(size(Sample));     
        
                 for ii = 1:K
                    It = Ig.*P(:,:,ii);
                    ft_It = ft2(It);
                    M(:,:,ii) = real(ift2(ft_It.*OTF_em)); % modeled images                  
                    r(:,:,ii) = D(:,:,ii)./M(:,:,ii);                    
                    FT_r = ft2(r(:,:,ii));
                    temp = real(ift2(FT_r.*OTF_em));
                    S = S + temp.*P(:,:,ii);  
                 end
                
         Ig_upd = Ig.*S; 
         
         err = abs(Ig_upd - Sample);
         abs_error(n) = mean(err(:));
         
         Ig = Ig_upd;
%          uncomment to see the result at intermediate iterations:
%          figure(); imagesc(Ig); axis square; colormap gray; title(['joint RL iteration = ' num2str(n)]);      
         
    end 

%---------------------------------------------------------------------
%------------------------Display results------------------------------
%---------------------------------------------------------------------
close all
figure(); imagesc(Ig); axis square; axis off; colormap gray; title('joint RL reconstruction');
figure(); imagesc(wf); axis square; axis off; colormap gray; title('Widefield');

figure(); imagesc(Ig(150:220, 75:145)); colormap gray; axis square; axis off; title('joint RL reconstruction, zoom 1');
figure(); imagesc(Ig(280:360, 360:440)); colormap gray; axis square; axis off; title('joint RL reconstruction, zoom 2');

disp(['total number of iterations = ' num2str(n)]);
disp(['pixel size in nm = ' num2str(L/N)]);
disp(['lambda/4NA in nm = ' num2str(lambda_em/4/NA)]);

